/*
Copyright by Dr. Bianca Hoffmann

Research Group Applied Systems Biology - Head: Prof. Dr. Marc Thilo
Figge
https://www.leibniz-hki.de/en/applied-systems-biology.html 
HKI-Center for Systems Biology of Infection
Leibniz Institute for Natural Product Research and Infection Biology -
Hans Knöll Insitute (HKI)
Adolf-Reichwein-Straße 23, 07745 Jena, Germany

*/

setBatchMode(true);
close("*");
roiManager("Reset");
run("Clear Results");

print("\\Clear");
	
run("Colors...", "foreground=white background=black selection=yellow");

home = getDirectory("Select folder with input data...");
out = getDirectory("Select folder for output data...");

if(!File.exists(out)){
	File.makeDirectory(out);
}

dirs = getFileList(home);

for(d = 0; d < dirs.length; d++){
	act_dir = dirs[d];
	
	files = getFileList(home + dirs[d]);
	for(f = 0; f < files.length; f++){
		if(endsWith(files[f], ".czi")){
			path_token = replace(dirs[d], "/","");
			processImage(home + dirs[d] + "\\" + files[f], path_token);
		}else{
			files2 = getFileList(home + dirs[d] + "\\" + files[f]);
			for(f2 = 0; f2 < files2.length; f2++){
				if(endsWith(files2[f2], ".czi")){
					path_token = replace(dirs[d], "/","") + "$" + replace(files[f], "/","");
					processImage(home + dirs[d] + "\\" + files[f] + "\\" + files2[f2], path_token);
				}
			}
		}
	}
}

function processImage(path, token){
	run("Bio-Formats Importer", "open=[" + path + "] autoscale color_mode=Colorized open_files rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
	title = getTitle();

	print("Pre-processing: " + title);

	setOption("ScaleConversions", true);
	run("8-bit");

	run("Duplicate...", "title=green duplicate channels=2");
	run("Enhance Contrast...", "saturated=0.1 normalize");
	run("Median...", "radius=5");
	
	selectWindow(title);
	run("Duplicate...", "title=collagen duplicate channels=3");
	run("Enhance Contrast...", "saturated=1 normalize");
	run("Median...", "radius=5");
	run("Morphological Filters", "operation=Closing element=Disk radius=3");
	rename("collagen2");
	run("Subtract Background...", "rolling=30");
	run("Median...", "radius=5");
	run("Enhance Local Contrast (CLAHE)", "blocksize=127 histogram=256 maximum=3 mask=*None* fast_(less_accurate)");
	
	selectWindow(title);
	run("Duplicate...", "title=red duplicate channels=4");
	run("Enhance Contrast...", "saturated=0.1 normalize");
	run("Median...", "radius=5");

	imageCalculator("Add", "green","red");
	imageCalculator("Subtract", "green","collagen2");

	selectWindow("green");
	saveAs("Tiff", out + token + "$" + replace(title, ".czi",""));

	close("*");
}