/*
Copyright by Dr. Bianca Hoffmann

Research Group Applied Systems Biology - Head: Prof. Dr. Marc Thilo
Figge
https://www.leibniz-hki.de/en/applied-systems-biology.html 
HKI-Center for Systems Biology of Infection
Leibniz Institute for Natural Product Research and Infection Biology -
Hans Knöll Insitute (HKI)
Adolf-Reichwein-Straße 23, 07745 Jena, Germany

*/

setBatchMode(true);
close("*");
roiManager("Reset");
run("Clear Results");
run("ROI Manager...");
print("\\Clear");
run("Set Measurements...", "mean min redirect=None decimal=3");

home = getDirectory("Select Cellpose output directory...");;
out = getDirectory("Select ROI output directory...");

if(!File.exists(out))
	File.makeDirectory(out);

files = getFileList(home);

for (f= 0; f < files.length; f++) {
	if(endsWith(files[f], ".tif")){
		print("Processing ROIs of " + files[f]);

		open(home + files[f]);
		title = getTitle();
		
		mask = home + replace(title, ".tif", "_cp_masks.png");

		open(mask);
		maskTitle = getTitle();

		roiManager("reset");

		selectWindow(maskTitle);

		run("Measure");
		min = 1;
		max = getResult("Max");

		for (i = min; i <= max; i++) {
			selectWindow(maskTitle);
			setThreshold(i, i);
			run("Analyze Particles...", "add");
		}
		
		count = roiManager("count");
		print("    " + count + " ROIs found");
		for (i = 0; i < count; i++) {
			roiManager("Select", i);
			run("Enlarge...", "enlarge=-1 pixel");
			roiManager("Update");
		}	

		roiManager("Save", out + replace(title, ".tif", "_rois.zip"));

		selectWindow(title);
		run("8-bit");
		run("From ROI Manager");
		run("Overlay Options...", "stroke=yellow width=2 fill=none apply show");
		run("Overlay Options...", "stroke=yellow width=2 fill=none apply show");
		
		saveAs("Png", out + replace(title, ".tif",""));

		close("*");
		roiManager("reset");
	}
}



