/*
Copyright by Dr. Bianca Hoffmann

Research Group Applied Systems Biology - Head: Prof. Dr. Marc Thilo
Figge
https://www.leibniz-hki.de/en/applied-systems-biology.html 
HKI-Center for Systems Biology of Infection
Leibniz Institute for Natural Product Research and Infection Biology -
Hans Knöll Insitute (HKI)
Adolf-Reichwein-Straße 23, 07745 Jena, Germany

*/


function getGreen(image){
	selectWindow(image);
	run("Select None");
	run("Duplicate...", "title=green duplicate channels=2");

	run("Measure");

	area_all = getResult("Area");
	
	run("8-bit");
	run("Enhance Contrast...", "saturated=0.1 normalize");
	run("k-means Clustering ...", "number_of_clusters=3 cluster_center_tolerance=0.000010000 enable_randomization_seed randomization_seed=48");
	rename("k-means");

	means = newArray(3);
	areas = newArray(3);
	minv = 255;
	min = -1;
	maxv = 0;
	max = -1;
	for (v = 0; v <= 2; v++) {
		selectWindow("k-means");
		run("Select None");
		setThreshold(v, v);		
		run("Create Selection");
		selectWindow("green");
		run("Restore Selection");
		run("Measure");
		means[v] = getResult("Mean");
		areas[v] = getResult("Area");
		run("Select None");
		if(means[v] < minv){
			min = v;
			minv = means[v];
		}
		if(means[v] > maxv){
			max = v;
			maxv = means[v];
		}
	}
	
	mid = -1;
	if(min != 0 && max != 0)
		mid = 0;
	if(min != 1 && max != 1)
		mid = 1;
	if(min != 2 && max != 2)
		mid = 2;	
	
	selectWindow("k-means");
	run("Select None");
	run("Duplicate...", "title=out");
	selectWindow("out");
	close("out");

	// use max and mid region if means ratio < 0
	mean_ratio = (means[max]/means[mid])-(means[mid]/means[min]);
	
	if(mean_ratio < 0){
		selectWindow("k-means");
		run("Duplicate...", "title=tmp");
		selectWindow("tmp");
		setThreshold(max, max);
		setOption("BlackBackground", true);
		run("Convert to Mask");

		selectWindow("k-means");
		setThreshold(mid, mid);
		setOption("BlackBackground", true);
		run("Convert to Mask");

		imageCalculator("Add", "k-means","tmp");
		close("tmp");
	}else{
		selectWindow("k-means");
		setThreshold(max, max);
		setOption("BlackBackground", true);
		run("Convert to Mask");
	}
	close("green");

	selectWindow("k-means");
	rename("green");
}

function getRed(image){
	selectWindow(image);
	run("Duplicate...", "title=red duplicate channels=4");
	selectWindow("red");
	run("Enhance Contrast...", "saturated=0.1 normalize");
	run("8-bit");
	run("BEEPS ", "range_filter=gauss photometric_standard_deviation=30.0 spatial_decay=0.01 iterations=1");
	run("Measure");
	mean = getResult("Mean");
	thresh = mean * red_thresh_percent;
	setThreshold(thresh, 255);
	run("Convert to Mask");
}

function getCollagen(image){
	selectWindow(image);
	run("Duplicate...", "title=collagen duplicate channels=3");
	selectWindow("collagen");
	run("8-bit");
	run("Enhance Contrast...", "saturated=0.1 normalize");
	run("Duplicate...","title=collagen_overlay");
	selectWindow("collagen");
	run("Median...", "radius=2");
	run("Subtract Background...", "rolling=150");
	setAutoThreshold("Triangle dark");
	run("Convert to Mask");
	run("Remove Outliers...", "radius=2 threshold=50 which=Bright");
	run("Create Selection");
	roiManager("Reset");
	roiManager("Add");
}


function processImage(folder, image, roi, fullname){
	run("Bio-Formats Importer", "open=[" + image + "] autoscale color_mode=Colorized open_files rois_import=[ROI manager] view=Hyperstack stack_order=XYCZT");
	t2 = getTitle();

	getPixelSize(unit, pixelWidth, pixelHeight);
	imageArea = getWidth() * getHeight() * pixelWidth * pixelHeight;

	setSlice(2);
	run("Green");
	
	setSlice(4);
	run("Red");

	getCollagen(t2);

	selectWindow("collagen_overlay");
	run("From ROI Manager");
	run("Overlay Options...", "stroke=#ffff00 width=2 fill=none set apply");
	run("Flatten");
	saveAs("PNG", out + replace(t2,".czi","_collagen_area.png"));
	rename("tmp");
	close("tmp");
	close("collagen_overlay");

	
	selectWindow("collagen");
	run("Create Selection");
	run("Measure");
	coll_area = getResult("Area");
	run("Make Inverse");
	run("Measure");
	cell_area = getResult("Area");
	run("Select None");

	roiManager("reset");
	close("collagen");
	
	getGreen(t2);
	getRed(t2);

	imageCalculator("AND create", "green","red");
	rename("hybrid");
	run("Fill Holes");
	run("Open");
	run("Remove Outliers...", "radius=5 threshold=50 which=Bright");
	
	selectWindow(t2);
	run("Select None");
	run("Duplicate...", "title=green2 duplicate channels=2");
	selectWindow(t2);
	run("Duplicate...", "title=red2 duplicate channels=4");

	run("Merge Channels...", "c1=green2 c2=red2 create");
	selectWindow("Composite");
	run("Select None");
	setSlice(1);
	run("Enhance Contrast...", "saturated=0.1");
	setSlice(2);
	run("Enhance Contrast...", "saturated=0.1");

	selectWindow(t2);
	run("Duplicate...", "title=overlay duplicate channels=3");
	run("8-bit");
	run("Enhance Contrast...", "saturated=0.1 normalize equalize");

	open(roi);

	for (i = 0; i < roiManager("count"); i++) {
		roiManager("Select", i);
		run("Enlarge...", "enlarge=-1 pixel");
		roiManager("Update");
	}
	roiManager("deselect");
//	roiManager("save", out +  replace(t2,".czi","_rois.zip"));
	
	if(roiManager("Count") > 0){
		run("Clear Results");
		roiManager("Measure");
		run("Summarize");
		
		selectWindow("green");
		width = getWidth();
		height = getHeight();
		newImage("red_cells", "8-bit black", width, height, 1);
		newImage("green_cells", "8-bit black", width, height, 1);
		newImage("hybrid_cells", "8-bit black", width, height, 1);
		newImage("fat_cells", "8-bit black", width, height, 1);

		nROI = roiManager("Count");
		run("Colors...", "foreground=white background=black selection=yellow");

		classification = newArray(nROI);
		
		num_cells = 0;
		num_border = 0;
		num_green = 0;
		num_red = 0;
		num_hybrid = 0;
		num_other = 0;
		mean_size = "NA";
		green_size = "NA";
		green_minSize = "NA";
		green_maxSize = "NA";
		green_perim = "NA";
		green_circ = "NA";
		green_feret = "NA";
		green_feretmin = "NA";
		green_ar = "NA";
		
		red_size = "NA";
		red_minSize = "NA";
		red_maxSize = "NA";
		red_perim = "NA";
		red_circ = "NA";
		red_feret = "NA";
		red_feretmin = "NA";
		red_ar = "NA";

		if(nROI > 0){
			selectWindow("Results");
			run("Close");
			roiManager("deselect");
			selectWindow("Composite");
			roiManager("Measure");
			run("Summarize");
			mean_size = getResult("Area", nResults - 4);

			selectWindow("Results");
			run("Close");
			
			for(r = 0; r < nROI; r++){
				selectWindow("green");
				roiManager("Select",r);
				roiManager("Measure");
				green_m = getResult("Mean");
	
				selectWindow("red");
				roiManager("Select",r);
				roiManager("Measure");
				red_m = getResult("Mean");

				area = getResult("Area");
				
				roiManager("select", r);
				Roi.getBounds(rx, ry, rw, rh);
				if(rx <= 2 || ry <= 2 || (rx + rw) >= (width-2) || (ry + rh) >= (height-2)){
					classification[r] = "b";
					num_border++;
				}else{
					num_cells++;
					if(green_m > green_cutoff && red_m > red_cutoff && area >= 100){
						selectWindow("hybrid_cells");
						roiManager("Select", r);
						roiManager("Fill");
						classification[r] = "h";
						num_hybrid++;
					}else{
						if(green_m > green_cutoff && area >= 100){
							selectWindow("green_cells");
							roiManager("Select", r);
							roiManager("Fill");
							classification[r] = "g";
							num_green++;
						}else{
							if(red_m > red_cutoff && area >= 100){
								selectWindow("red_cells");
								roiManager("Select", r);
								roiManager("Fill");
								classification[r] = "r";
								num_red++;
							}else{
								if(area >= 100){
									selectWindow("fat_cells");
									roiManager("Select", r);
									roiManager("Fill");
									classification[r] = "o";
									num_other++;
								}else{
									classification[r] = "b";
									num_border++;
								}
								
							}
						}
					}
				}
			}
		}

		roiManager("deselect");

		run("Clear Results");
		
		selectWindow("Composite");
		roiManager("Show All without labels");
		roiManager("measure");

		for (i = 0; i < classification.length; i++) {
			string = "" + replace(folder,"/","") + ";" + t2 + ";" + imageArea + ";" + coll_area + ";" + (i+1) + ";" + classification[i] + ";" + getResult("Area", i) + ";" + getResult("Perim.", i) + ";" + getResult("Circ.", i) + ";" + getResult("Feret", i) + ";" + getResult("MinFeret", i) + ";" + getResult("AR", i);
			File.append(string, out_file);
		}
		selectWindow("Composite");
		roiManager("show none");
		run("Remove Overlay");

		selectWindow("green_cells");
		run("Canvas Size...", "width=" + (width - 2) + " height=" + (height -2) + " position=Center");
		run("Properties...", "channels=1 slices=1 frames=1 unit=" + unit + " pixel_width=" + pixelWidth + " pixel_height=" + pixelHeight + " voxel_depth=1");

		selectWindow("red_cells");
		run("Canvas Size...", "width=" + (width - 2) + " height=" + (height -2) + " position=Center");
		run("Properties...", "channels=1 slices=1 frames=1 unit=" + unit + " pixel_width=" + pixelWidth + " pixel_height=" + pixelHeight + " voxel_depth=1");

		selectWindow("hybrid_cells");
		run("Canvas Size...", "width=" + (width - 2) + " height=" + (height -2) + " position=Center");
		run("Properties...", "channels=1 slices=1 frames=1 unit=" + unit + " pixel_width=" + pixelWidth + " pixel_height=" + pixelHeight + " voxel_depth=1");

		selectWindow("fat_cells");
		run("Canvas Size...", "width=" + (width - 2) + " height=" + (height -2) + " position=Center");
		run("Properties...", "channels=1 slices=1 frames=1 unit=" + unit + " pixel_width=" + pixelWidth + " pixel_height=" + pixelHeight + " voxel_depth=1");

		selectWindow("green_cells");
		roiManager("reset");
		run("Analyze Particles...", "size=100-Infinity add");
		selectWindow("Composite");
		run("Select None");
		if(roiManager("Count") > 0){
			selectWindow("Results");
			run("Close");
			selectWindow("green_cells");
			roiManager("Measure");
			run("Summarize");
			if(nResults > 1){
				green_size = getResult("Area", nResults - 4);
				green_minSize = getResult("Area", nResults - 2);
				green_maxSize = getResult("Area", nResults - 1);

				green_perim = getResult("Perim.", nResults - 4);
				green_circ = getResult("Circ.", nResults - 4);
				green_feret = getResult("Feret", nResults - 4);
				green_feretmin = getResult("MinFeret", nResults - 4);
				green_ar = getResult("AR", nResults - 4);
			}
			else{
				green_size = getResult("Area");
				green_minSize = green_size;
				green_maxSize = green_size;
				
				green_perim = getResult("Perim.");
				green_circ = getResult("Circ.");
				green_feret = getResult("Feret");
				green_feretmin = getResult("MinFeret");
				green_ar = getResult("AR");
			}
			
			selectWindow("Composite");
			run("From ROI Manager");
			run("Overlay Options...", "stroke=green width=3 fill=none set apply");
			run("Overlay Options...", "stroke=green width=2 fill=none set apply");
			run("Flatten");
			rename("flat");
			close("Composite");
			selectWindow("flat");
			rename("Composite");
		}

		selectWindow("red_cells");
		roiManager("reset");
		run("Analyze Particles...", "size=100-Infinity add");
		selectWindow("Composite");
		if(roiManager("Count") > 0){
			selectWindow("Results");
			run("Close");
			selectWindow("green_cells");
			roiManager("Measure");
			run("Summarize");
			if(nResults > 1){
				red_size = getResult("Area", nResults - 4);
				red_minSize = getResult("Area", nResults - 2);
				red_maxSize = getResult("Area", nResults - 1);
	
				red_perim = getResult("Perim.", nResults - 4);
				red_circ = getResult("Circ.", nResults - 4);
				red_feret = getResult("Feret", nResults - 4);
				red_feretmin = getResult("MinFeret", nResults - 4);
				red_ar = getResult("AR", nResults - 4);
			}
			else{
				red_size = getResult("Area");
				red_minSize = redn_size;
				red_maxSize = red_size;
					
				red_perim = getResult("Perim.");
				red_circ = getResult("Circ.");
				red_feret = getResult("Feret");
				red_feretmin = getResult("MinFeret");
				red_ar = getResult("AR");
			}
			
			selectWindow("Composite");
			run("From ROI Manager");
			run("Overlay Options...", "stroke=red width=3 fill=none set apply");
			run("Overlay Options...", "stroke=red width=2 fill=none set apply");
			run("Flatten");
			rename("flat");
			close("Composite");
			selectWindow("flat");
			rename("Composite");
		}

		selectWindow("hybrid_cells");
		roiManager("reset");
		run("Analyze Particles...", "size=100-Infinity add");
		selectWindow("Composite");
		if(roiManager("Count") > 0){
			run("From ROI Manager");
			run("Overlay Options...", "stroke=orange width=3 fill=none set apply");
			run("Overlay Options...", "stroke=orange width=2 fill=none set apply");
			run("Flatten");
			rename("flat");
			close("Composite");
			selectWindow("flat");
			rename("Composite");
		}

		selectWindow("fat_cells");
		roiManager("reset");
		run("Analyze Particles...", "size=100-Infinity add");
		selectWindow("Composite");
		if(roiManager("Count") > 0){
			run("From ROI Manager");
			run("Overlay Options...", "stroke=white width=3 fill=none set apply");
			run("Overlay Options...", "stroke=white width=2 fill=none set apply");
			run("Flatten");
			rename("flat");
			close("Composite");
			selectWindow("flat");
			rename("Composite");
		}

		selectWindow("Composite");
		saveAs("PNG", out + replace(t2,".czi","_fibre_classification.png"));
		rename("Composite");
		
		roiManager("reset");
		selectWindow("green_cells");
		run("Analyze Particles...", "size=100-Infinity add");
		selectWindow("red_cells");
		run("Analyze Particles...", "size=100-Infinity add");
		selectWindow("hybrid_cells");
		run("Analyze Particles...", "size=100-Infinity add");
		selectWindow("fat_cells");
		run("Analyze Particles...", "size=100-Infinity add");
		
		roiManager("deselect");
		roiManager("save", out +  replace(t2,".czi","_rois_noBorder.zip"));
		
	}else{
		selectWindow("Composite");
		saveAs("PNG", out + replace(t2,".czi","_fibre_classification.png"));
	}
	
	string = "" + replace(folder,"/","") + ";" + t2 + ";" + imageArea + ";" + coll_area + ";" + num_green + ";" + num_red + ";" + num_hybrid + ";" + num_other + ";" + mean_size + ";" + green_size + ";" + red_size + ";" + green_minSize + ";" + green_maxSize + ";" + red_minSize + ";" + red_maxSize + ";" + green_perim + ";" + green_circ + ";" + green_feret + ";" + green_feretmin + ";" + green_ar+ ";" + red_perim + ";" + red_circ + ";" + red_feret + ";" + red_feretmin + ";" + red_ar;
	print("   Output string: " + string);
	
	File.append(string, out_file2);

	
	close("*");
	roiManager("Reset");

}

setBatchMode(true);
close("*");
roiManager("Reset");
run("Clear Results");
run("ROI Manager...");

red_thresh_percent = 0.9;
green_cutoff = 102;
red_cutoff = 102;

print("\\Clear");
	
run("Set Measurements...", "area mean standard min perimeter shape feret's redirect=None decimal=2");
run("Colors...", "foreground=white background=black selection=yellow");

home = getDirectory("Select folder with input data...");
rois = getDirectory("Select folder with ROIs...");

out = getDirectory("Select output folder...");

if(!File.exists(out))
	File.makeDirectory(out);

out_file = File.open(out + "Classification_results_greenCutoff-" + green_cutoff + "_redCutoff-" + red_cutoff + "_red_thresh-" + red_thresh_percent + "_singleFibres.txt");
print(out_file, "Folder;Image;ImageArea;CollagenArea;FiberID;FiberType;Area;Perimeter;Circularity;Feret;FeretMin;AspectRatio");
File.close(out_file);
out_file = out + "Classification_results_greenCutoff-" + green_cutoff + "_redCutoff-" + red_cutoff + "_red_thresh-" + red_thresh_percent + "_singleFibres.txt";

out_file2 = File.open(out + "Classification_results_greenCutoff-" + green_cutoff + "_redCutoff-" + red_cutoff + "_red_thresh-" + red_thresh_percent + ".txt");
print(out_file2, "Folder;Image;ImageArea;CollagenArea;Green;Red;Hybrid;Other;Size;GreenSize;RedSize;GreenMinSize;GreenMaxSize;RedMinSize;RedMaxSize;GreenPerimeter;GreenCircularity;GreenFeret;GreenFeretMin;GreenAspectRatio;RedPerimeter;RedCircularity;RedFeret;RedFeretMin;RedAspectRatio");
File.close(out_file2);
out_file2 = out + "Classification_results_greenCutoff-" + green_cutoff + "_redCutoff-" + red_cutoff + "_red_thresh-" + red_thresh_percent + ".txt";
dirs = getFileList(home);

for(d = 0; d < dirs.length; d++){
	act_dir = dirs[d];

	files = getFileList(home + dirs[d]);
	for(f = 0; f < files.length; f++){
	
		if(endsWith(files[f], ".czi")){
			fullname = replace(dirs[d],"/","") + "$" + replace(files[f], ".czi", "");
			print("Processing " + fullname);
			roi = rois + replace(dirs[d],"/","") + "$" + replace(files[f], ".czi", "_rois.zip");
			print("   ROI file: " + roi);
			
			processImage(dirs[d], home + dirs[d] + "\\" + files[f], roi, fullname);
		}else{
			files2 = getFileList(home + dirs[d] + "\\" + files[f]);
			for(f2 = 0; f2 < files2.length; f2++){
				if(endsWith(files2[f2], ".czi")){
					fullname = replace(dirs[d],"/","") + "$" + replace(files[f],"/","") + "$" + replace(files2[f2], ".czi", "");
					
					print("Processing " + fullname);
					roi = rois + replace(dirs[d],"/","") + "$" + replace(files[f],"/","") + "$" + replace(files2[f2], ".czi", "_rois.zip");
					print("   ROI file: " + roi);
					
					processImage(files[f], home + dirs[d] + "\\" + files[f] + "\\" + files2[f2], roi, fullname);
				}
			}
		}
	}
}




